/* Limit the height of the typeahead list */
.typeahead__list {
    max-height: 24em;
    overflow: auto;
}

/* Fix location of spinning wheel */
.typeahead__container.loading .typeahead__query::after,
.typeahead__container.loading .typeahead__query::before {
    top: 0.5rem;
}

/* Fix lense position */
.typeahead__button button {
    height: 100%;
}

/* Fix color for DarkMode */
.typeahead__container button,
.typeahead__container button.disabled,
.typeahead__container button[disabled],
.typeahead__container input.disabled,
.typeahead__container input[disabled],
.typeahead__container.hint .typeahead__field [contenteditable],
.typeahead__container.hint .typeahead__field input,
.typeahead__container.hint .typeahead__field textarea
.typeahead__dropdown,
.typeahead__field .typeahead__hint,
.typeahead__field [contenteditable],
.typeahead__field input,
.typeahead__field textarea,
.typeahead__list {
    background-color: var(--bs-body-bg);
}

.typeahead__dropdown .typeahead__dropdown-item>a,
.typeahead__list .typeahead__item>a,
.typeahead__list.empty>li {
    color: var(--bs-body-color);
}

.typeahead__dropdown .typeahead__dropdown-item>a>strong,
.typeahead__list .typeahead__item>a>strong {
    color: var(--bs-emphasis-color);
}

.typeahead__dropdown .typeahead__dropdown-item:not([disabled]).active>a,
.typeahead__dropdown .typeahead__dropdown-item:not([disabled])>a:focus,
.typeahead__dropdown .typeahead__dropdown-item:not([disabled])>a:hover,
.typeahead__list .typeahead__item:not([disabled]).active>a,
.typeahead__list .typeahead__item:not([disabled])>a:focus,
.typeahead__list .typeahead__item:not([disabled])>a:hover {
    background-color: var(--bs-tertiary-bg);
    color: var(--bs-body-color);
}