<html lang="{{ get_translation().locale }}">
    <head>
        <title>test-jinja2</title>
    </head>
    <body>
        {{ get_language_name(get_translation().locale) }}<br/>
        {% trans %}Some text to translate{% endtrans %}<br/>
        {{ my_datetime | format_datetime(format='full') }}<br/>
        {{ my_date | format_date(format='full') }}
    </body>
</html>
