{# def messages, floating=False #}
<div {{ attrs.render(class="flash-messages") }}>
  {% for message, level in messages %}
    <div class="alert alert-{{'danger' if level == 'error' else level }} alert-dismissible fade show"
         role="alert">
      {{ message }}
      <button type="button"
              class="btn-close"
              data-bs-dismiss="alert"
              aria-label="Close"></button>
    </div>
  {% endfor %}
</div>
